#include "stdio.h"
#include "protein.h"
#include "rw_energy.h"
#include "scb_energy.h"
#include "protein_data.h"
#include "memory.h"
#include "error.h"
#include "string.h"
#include "math.h"
#include <vector>
#include <iostream>

using namespace REFINE_NS;
#define CONTACTDISTANCE 8.0


/* ---------------------------------------------------------------------- 
     calculate RW potential : all atom pair-wise statistical potential 
     with random walk chain reference state
   ---------------------------------------------------------------------- */
double Protein::calculate_rw()
{
	//printf("J000 %d %d\n", Taa, Tatom);
	int i, j, ii, jj, kk;
	double dist, CUT = 15.499, result = 0.0, DELTA = 0.5;

	// calculate reference state if not
	if (rw_flag == 0)
	{
		calculate_rw_reference();
	}

	for (i = 0; i < Tatom; i++)
	{
		for (j = i + 1; j < Tatom; j++)
		{
			if (naa[i] == naa[j])
			{
				continue;
			}
			dist = sqrt((x[i][0] - x[j][0]) * (x[i][0] - x[j][0]) + (x[i][1] - x[j][1]) * (x[i][1] - x[j][1]) + (x[i][2] - x[j][2]) * (x[i][2] - x[j][2]));
			if (dist > CUT)
			{
				continue;
			}
			ii = (int)(dist / DELTA);
			jj = type158[i];
			kk = type158[j];
			if (jj > 159 || kk > 159)
			{
				continue;
			}
			if (ii < 0)
				result += 0;
			else
				result += rwp[jj][kk][ii];
			//printf("J0002 %d %d %d %d %f\n",ii,jj,kk,rw[ii][jj-1][kk-1],rwp[jj][kk][ii]);
		}
	}
	return (result);
}

/* ---------------------------------------------------------------------- 
     calculate idea random walk chain reference state
   ---------------------------------------------------------------------- */
void Protein::calculate_rw_reference()
{
	int i, j, k, BIN = 31;
	double r, ref[40], b2 = 460;

	/* calculate reference state                 */
	for (i = 0; i < BIN; i++)
	{
		r = 0.5 * i + 0.25;
		ref[i] = 0.0;
		for (j = 1; j < Taa; j++)
		{
			ref[i] += r * r * pow((1.0 / j), 1.5) * exp(-1.5 * r * r / (j * b2));
			//printf("%d %d %f\n",i,j,ref[i]);
		}
	}
	/* calculate the KB potential  matrix                                */
	for (k = 0; k < BIN; k++)
	{
		for (j = 1; j < 159; j++)
		{
			for (i = j; i < 159; i++)
			{
				if (rw[k][i - 1][j - 1] == 0 && rw[k][j - 1][i - 1] == 0)
				{
					rwp[i][j][k] = 10.0;
					rwp[j][i][k] = 10.0;
					// 	   fprintf(out,"%5d %5d %5d %15.6f \n",i,j,k,kbp[i][j][k]);
				}
				else
				{
					rwp[i][j][k] = -log((rw[k][i - 1][j - 1] + rw[k][j - 1][i - 1]) * ref[BIN - 1] / (rw[BIN - 1][i - 1][j - 1] + rw[BIN - 1][j - 1][i - 1]) / ref[k]);
					// 	   fprintf(out,"%5d %5d %5d %15.6f \n",i,j,k,kbp[i][j][k]);
					rwp[j][i][k] = rwp[i][j][k];
				}
				//if(j==6&&i==138){
				//printf("%d %d %d %d %f\n",i,j,k,rw[k][i-1][j-1],rwp[i][j][k]);
				//}
			}
		}
	}
	//memory->sfree(ref);
	rw_flag = 1;
}

/* ----------------------------------------------------------------------
     calculate side-chain orientation statistical potential
   ---------------------------------------------------------------------- */
double Protein::calculate_scb()
{

	int nb, p, i, j, k, b1, b2, good1[8], bb[6000], kk, binnum;
	double px1, py1, pz1, px2, py2, pz2, qx1, qy1, qz1, qx2, qy2, qz2, dist, result = 0;
	double rsqq1, rsqq2, rsqp1, rsqp2, rp1, rp2, rq1, rq2;
	double theta1, theta2, phi1, phi2, c, n1[3], n2[3];
	double np[3], nq[3], rnp, rnq, cd, psi, dv, rn1, rn2, p0[3], q0[3], pi = PI, piinv = 1.0 / PI;
	int atomp[3], atomq[3], atompp[18000];

	// calculate reference state if not
	if (scb_flag == 0)
	{
		calculate_scb_reference();
	}

	// Step through all the atoms and get the block array
	nb = 0;
	for (i = 0; i < 8; i++)
	{
		good1[i] = 0;
	}
	for (p = 0; p < Tatom; p++)
	{
		b1 = side_chain_block_type(AA3[p]);
		b2 = block_type(AA3[p]);
		if (b1 > 18)
		{
			continue;
		}
		if (b2 == 5)
		{
			for (i = 0; i < 3; i++)
			{
				good1[i] = 1;
			}
			good1[6] = 1;
		}
		if (b2 == 0)
		{
			for (i = 0; i < 3; i++)
			{
				good1[i] = 1;
			}
		}
		for (j = 0; j < 3; j++)
		{
			if (Tab6[b1][j + 1] == type[p])
			{
				atompp[nb * 3 + 6 + j] = p;
				good1[j] = 1;
			}
		}
		if (type[p] == 0)
		{
			atompp[nb * 3] = p;
			good1[3] = 1;
		}
		if (type[p] == 1)
		{
			atompp[nb * 3 + 1] = p;
			atompp[nb * 3 + 3] = p;
			good1[4] = 1;
		}
		if (type[p] == 26)
		{
			atompp[nb * 3 + 2] = p;
			good1[5] = 1;
		}
		if (type[p] == 2)
		{
			atompp[nb * 3 + 4] = p;
			good1[6] = 1;
		}
		if (type[p] == 17)
		{
			atompp[nb * 3 + 5] = p;
			good1[7] = 1;
		}
		if (good1[0] != 1 || good1[1] != 1 || good1[2] != 1 || good1[3] != 1 || good1[4] != 1 || good1[5] != 1 || good1[6] != 1 || good1[7] != 1)
		{
			continue;
		}
		else
		{
			for (i = 0; i < 8; i++)
			{
				good1[i] = 0;
			}
			if (b2 == 5)
			{
				bb[nb] = 18;
				nb++;
			}
			else if (b2 == 0)
			{
				bb[nb] = 18;
				bb[nb + 1] = 19;
				nb = nb + 2;
			}
			else
			{
				bb[nb] = 18;
				bb[nb + 1] = 19;
				bb[nb + 2] = b1;
				nb = nb + 3;
			}
		}
	}

	// loop all blocks
	for (i = 0; i < nb; i++)
	{
		for (j = 0; j < nb; j++)
		{
			for (k = 0; k < 3; k++)
			{
				atomp[k] = atompp[i * 3 + k];
				atomq[k] = atompp[j * 3 + k];
			}
			b1 = bb[i];
			b2 = bb[j];
			// got two blocks and calculate the orientation
			dist = sqrt((x[atomp[0]][0] - x[atomq[0]][0]) * (x[atomp[0]][0] - x[atomq[0]][0]) +
						(x[atomp[0]][1] - x[atomq[0]][1]) * (x[atomp[0]][1] - x[atomq[0]][1]) +
						(x[atomp[0]][2] - x[atomq[0]][2]) * (x[atomp[0]][2] - x[atomq[0]][2]));
			// distance cut-off 10 Angstrom
			if (dist > 10.0 || dist < 3.0)
			{
				continue;
			}
			// vector O->O'
			p0[0] = x[atomq[0]][0] - x[atomp[0]][0];
			p0[1] = x[atomq[0]][1] - x[atomp[0]][1];
			p0[2] = x[atomq[0]][2] - x[atomp[0]][2];

			// vector O'->O
			q0[0] = -p0[0];
			q0[1] = -p0[1];
			q0[2] = -p0[2];

			// vector p1
			px1 = x[atomp[1]][0] - x[atomp[0]][0];
			py1 = x[atomp[1]][1] - x[atomp[0]][1];
			pz1 = x[atomp[1]][2] - x[atomp[0]][2];
			rsqp1 = px1 * px1 + py1 * py1 + pz1 * pz1;
			rp1 = sqrt(rsqp1);

			// vector p2
			px2 = x[atomp[2]][0] - x[atomp[0]][0];
			py2 = x[atomp[2]][1] - x[atomp[0]][1];
			pz2 = x[atomp[2]][2] - x[atomp[0]][2];
			rsqp2 = px2 * px2 + py2 * py2 + pz2 * pz2;
			rp2 = sqrt(rsqp2);

			// vector q1
			qx1 = x[atomq[1]][0] - x[atomq[0]][0];
			qy1 = x[atomq[1]][1] - x[atomq[0]][1];
			qz1 = x[atomq[1]][2] - x[atomq[0]][2];
			rsqq1 = qx1 * qx1 + qy1 * qy1 + qz1 * qz1;
			rq1 = sqrt(rsqq1);

			// vector q2
			qx2 = x[atomq[2]][0] - x[atomq[0]][0];
			qy2 = x[atomq[2]][1] - x[atomq[0]][1];
			qz2 = x[atomq[2]][2] - x[atomq[0]][2];
			rsqq2 = qx2 * qx2 + qy2 * qy2 + qz2 * qz2;
			rq2 = sqrt(rsqq2);

			// normal vector n1, n2
			n1[0] = py1 * pz2 - pz1 * py2;
			n1[1] = pz1 * px2 - px1 * pz2;
			n1[2] = px1 * py2 - py1 * px2;
			rn1 = sqrt(n1[0] * n1[0] + n1[1] * n1[1] + n1[2] * n1[2]);

			n2[0] = qy1 * qz2 - qz1 * qy2;
			n2[1] = qz1 * qx2 - qx1 * qz2;
			n2[2] = qx1 * qy2 - qy1 * qx2;
			rn2 = sqrt(n2[0] * n2[0] + n2[1] * n2[1] + n2[2] * n2[2]);

			// normal vector of p0 and n1
			np[0] = n1[1] * p0[2] - n1[2] * p0[1];
			np[1] = n1[2] * p0[0] - n1[0] * p0[2];
			np[2] = n1[0] * p0[1] - n1[1] * p0[0];
			rnp = sqrt(np[0] * np[0] + np[1] * np[1] + np[2] * np[2]);

			// normal vector of q0 and n2
			nq[0] = n2[1] * q0[2] - n2[2] * q0[1];
			nq[1] = n2[2] * q0[0] - n2[0] * q0[2];
			nq[2] = n2[0] * q0[1] - n2[1] * q0[0];
			rnq = sqrt(nq[0] * nq[0] + nq[1] * nq[1] + nq[2] * nq[2]);

			// angle between p0 and n1
			c = p0[0] * n1[0] + p0[1] * n1[1] + p0[2] * n1[2];
			c /= dist * rn1;
			if (c > 1.0)
				c = 1.0;
			if (c < -1.0)
				c = -1.0;
			theta1 = acos(c);

			// angle between q0 and n2
			c = q0[0] * n2[0] + q0[1] * n2[1] + q0[2] * n2[2];
			c /= dist * rn2;
			if (c > 1.0)
				c = 1.0;
			if (c < -1.0)
				c = -1.0;
			theta2 = acos(c);

			// angle between p1 and np
			c = px1 * np[0] + py1 * np[1] + pz1 * np[2];
			c /= rp1 * rnp;
			if (c > 1.0)
				c = 1.0;
			if (c < -1.0)
				c = -1.0;
			phi1 = acos(c);

			// direction vector p1 * p0
			dv = px1 * p0[0] + py1 * p0[1] + pz1 * p0[2];
			if (dv < 0.0)
				phi1 = -phi1;

			// angle between q1 and nq
			c = qx1 * nq[0] + qy1 * nq[1] + qz1 * nq[2];
			c /= rq1 * rnq;
			if (c > 1.0)
				c = 1.0;
			if (c < -1.0)
				c = -1.0;
			phi2 = acos(c);

			// direction vector q1 * q0
			dv = qx1 * q0[0] + qy1 * q0[1] + qz1 * q0[2];
			if (dv < 0.0)
				phi2 = -phi2;

			// dihedral angle n1-OO'-n2
			cd = np[0] * nq[0] + np[1] * nq[1] + np[2] * nq[2];
			cd /= rnp * rnq;
			if (cd >= 1.0)
				cd = 0.999999;
			if (cd <= -1.0)
				cd = -0.999999;
			psi = acos(cd);

			// direction vector n1 * nq
			dv = n1[0] * nq[0] + n1[1] * nq[1] + n1[2] * nq[2];
			if (dv < 0.0)
				psi = -psi;
			psi = (psi > 0.0) ? pi - psi : -(pi + psi);

			// theta and phi, 26 bins

			// dihedral angle psi, pi/4 bin, 4 bins
			if (psi < -0.75 * pi || psi > 0.75 * pi)
			{
				kk = 3;
			}
			else
			{
				kk = (int)(psi * 2.0 * piinv + 1.5);
			}
			//printf("JZ1011 %f %f %d\n",theta1,phi1,kk);
			binnum = scb_bin_num(theta1, phi1) * 26 * 4 + scb_bin_num(theta2, phi2) * 4 + kk;
			result += scbp[b1][b2][binnum];
			//printf("JZ1002 %d %d %d %f\n",b1,b2,binnum,scbp[b1][b2][binnum]);
		}
	}
	return (result);
}
//---------------------------------------------------------------------- /
int Protein::block_type(char *resname)
{
	int i;
	for (i = 0; i < 20; i++)
	{
		if (!strncmp(sTab3[i], resname, 8))
		{
			return (i);
		}
	}
	return (99);
}
/* ---------------------------------------------------------------------- 
     calculate side-chain orientatio reference state
   ---------------------------------------------------------------------- */
void Protein::calculate_scb_reference()
{
	int i, j, k, kk, jj1, jj2, tmp;
	double tmp1;

	//scbp = memory->create_3d_double_array(20,20,2704,"Protein:scbp");
	// calculate sidechain block orientation potential

	for (j = 0; j < 20; j++)
	{
		for (i = 0; i < 20; i++)
		{
			tmp = 0;
			for (jj1 = 0; jj1 < 26; jj1++)
			{
				for (jj2 = 0; jj2 < 26; jj2++)
				{
					for (kk = 0; kk < 4; kk++)
					{
						k = jj1 * 26 * 4 + jj2 * 4 + kk;
						tmp += scb[k][i][j];
					}
				}
			}
			for (jj1 = 0; jj1 < 26; jj1++)
			{
				for (jj2 = 0; jj2 < 26; jj2++)
				{
					for (kk = 0; kk < 4; kk++)
					{
						k = jj1 * 26 * 4 + jj2 * 4 + kk;
						if (scb[k][j][i] == 0)
						{
							scbp[i][j][k] = 2.0;
						}
						else
						{
							//numcount[i][j][k]=numcount[i][j][k];
							tmp1 = tmp;
							if (jj1 == 0 || jj1 == 25)
							{
								tmp1 *= 0.5 * (1.0 - cos(PI / 6.0));
							}
							if ((jj1 > 0 && jj1 < 9) || (jj1 > 16 && jj1 < 25))
							{
								tmp1 *= (cos(PI / 6.0) - cos(PI / 3.0)) / 16.0;
							}
							if (jj1 > 8 && jj1 < 17)
							{
								tmp1 *= cos(PI / 3.0) / 16.0;
							}
							if (jj2 == 0 || jj2 == 25)
							{
								tmp1 *= 0.5 * (1.0 - cos(PI / 6.0));
							}
							if ((jj2 > 0 && jj2 < 9) || (jj2 > 16 && jj2 < 25))
							{
								tmp1 *= (cos(PI / 6.0) - cos(PI / 3.0)) / 16.0;
							}
							if (jj2 > 8 && jj2 < 17)
							{
								tmp1 *= cos(PI / 3.0) / 16.0;
							}
							scbp[i][j][k] = -log(4.0 * (0.1 + scb[k][j][i]) / tmp1);
							//printf("%d %d %d %d %f\n",i,j,k,scb[k][j][i],scbp[j][i][k]);
						}
					}
				}
			}
		}
	}
	scb_flag = 1;
}
//---------------------------------------------------------------------- /
int Protein::side_chain_block_type(char *resname)
{
	int i, j;
	for (i = 0; i < 20; i++)
	{
		if (!strncmp(sTab3[i], resname, 8))
		{
			for (j = 0; j < 18; j++)
			{
				if (Tab6[j][0] == i)
				{
					return (j);
				}
			}
		}
	}
	return (99);
}

//---------------------------------------------------------------------- /
int Protein::scb_bin_num(double theta, double phi)
{
	double pi = PI;
	double piinv = 1.0 / PI;
	int i, kk;
	if (phi < -0.875 * pi || phi > 0.875 * pi)
	{
		kk = 7;
	}
	else
	{
		kk = (int)(phi * 4.0 * piinv + 3.5);
	}
	if (theta < pi / 6.0)
	{
		i = 0;
	}
	else if (theta < pi / 3.0)
	{
		i = 1 + kk;
	}
	else if (theta < pi * 2.0 / 3.0)
	{
		i = 9 + kk;
	}
	else if (theta < pi * 5.0 / 6.0)
	{
		i = 17 + kk;
	}
	else
	{
		i = 25;
	}
	return (i);
}

/* ---------------------------------------------------------------------- */
Protein::Protein(REFINE *refine) : Pointers(refine)
{
	Taa = -1;	  // total number of amino acids
	Tatom = -1;	// total number of atoms of protein
	natom1 = NULL; // atom number of the first atom(N) in one amino acid

	AtomType = NULL;				  // type of atom
	AA3 = NULL;						  // type of amino acid
	naa = NULL;						  // residue number of this atom
	x = NULL;						  // x y z coordinates
	x0 = NULL;						  // x y z coordinates
	x00 = NULL;						  // x y z coordinates
	phi = NULL;						  // phi list
	psi = NULL;						  // psi list
	nclash = NULL;					  // number of atomic clash for one residue
	bondout = angleout = ramaout = 0; // of bond, angle, rama outliers
	bondE = angleE = ramaE = 0.0;	 // energy of bond, angle and rama
	//RandomSeed=1000;      // Random Seed
	rwp = NULL;   // rw potential
	scbp = NULL;  // side-chain orientation potential
	rw_flag = 0;  // flag of rw reference state calculation
	scb_flag = 0; // flag of side-chain orientation reference state calculation

	ntm = 0;			   // number tmhelix
	tmb = NULL;			   // beginning of tmhelix
	tme = NULL;			   // ending of tmhelix
	tmi = NULL;			   // index of tmhelix
	tmbnative = NULL;	  // beginning of tmhelix for native
	tmenative = NULL;	  // ending of tmhelix for native
	tminative = NULL;	  // index of tmhelix for native
	hydrophobicity = NULL; // hydrophobicity scale
	tmdv = NULL;		   // direction vector of TM helix

	nnlist = NULL; // neighbor list
	nlist = NULL;  // neighbor list

	sigma = NULL;   // for L-J potential
	epsilon = NULL; // for L-J potential
	type = NULL;	// atom type of sTab4[][8]
	type158 = NULL; // atom type 158 for RW

	sasa = NULL;	// solvent accessible surface area (sasa)
	nsalist = NULL; // sasa neighbor list
	salist = NULL;  // sasa neighbor list

	D100 = 100.0 * PI / 180.0; //100 degree/180 degree * PI

	ncontact = 0; // # of contact
	icontact = NULL;
	rcontact = NULL;
	icontacttype = NULL;
	confidence_contact = NULL;
	ncontacthelixmap = NULL;
	confidence_helix = NULL;
	nresiduecontact = NULL;

	ncontact_loop = 0; // # of loop contact
	icontact_loop = NULL;
	confidence_contact_loop = NULL;
	icontacttype_loop = NULL;

	helixconnectionAvg = NULL;
	helixconnectionDev = NULL;
	helixangleAvg = NULL;
	helixangleDev = NULL;

	loopfrag = NULL;
	loopfrag_number = NULL;

	init();
}

/* ---------------------------------------------------------------------- */
Protein::~Protein()
{
	if (natom1 != NULL)
	{
		natom1 = NULL;
		memory->sfree(natom1);
	}
	if (AtomType != NULL)
	{
		AtomType = NULL;
		memory->destroy_2d_char_array(AtomType);
	}
	if (AA3 != NULL)
	{
		AA3 = NULL;
		memory->destroy_2d_char_array(AA3);
	}
	if (x0 != NULL)
	{
		x0 = NULL;
		memory->sfree(naa);
	}
	if (type158 != NULL)
	{
		type158 = NULL;
		memory->destroy_2d_double_array(x);
	}
	if (type158 != NULL)
	{
		type158 = NULL;
		memory->destroy_2d_double_array(x0);
	}
	if (x00 != NULL)
	{
		x00 = NULL;
		memory->destroy_2d_double_array(x00);
	}
	if (phi != NULL)
	{
		phi = NULL;
		memory->sfree(phi);
	}
	if (psi != NULL)
	{
		psi = NULL;
		memory->sfree(psi);
	}
	if (nclash != NULL)
	{
		nclash = NULL;
		memory->sfree(nclash);
	}
	if (rwp != NULL)
	{
		rwp = NULL;
		memory->destroy_3d_double_array(rwp);
	}
	if (scbp != NULL)
	{
		scbp = NULL;
		memory->destroy_3d_double_array(scbp);
	}
	if (tmb != NULL)
	{
		tmb = NULL;
		memory->sfree(tmb);
	}
	if (tme != NULL)
	{
		tme = NULL;
		memory->sfree(tme);
	}
	if (tmi != NULL)
	{
		tmi = NULL;
		memory->sfree(tmi);
	}
	if (membren_vector != NULL)
	{
		membren_vector = NULL;
		memory->sfree(membren_vector);
	}

	if (tmbnative != NULL)
	{
		tmbnative = NULL;
		memory->sfree(tmbnative);
	}
	if (tmenative != NULL)
	{
		tmenative = NULL;
		memory->sfree(tmenative);
	}
	if (tminative != NULL)
	{
		tminative = NULL;
		memory->sfree(tminative);
	}
	if (hydrophobicity != NULL)
	{
		hydrophobicity = NULL;
		memory->sfree(hydrophobicity);
	}
	if (tmdv != NULL)
	{
		tmdv = NULL;
		memory->sfree(tmdv);
	}
	if (nnlist != NULL)
	{
		nnlist = NULL;
		memory->sfree(nnlist);
	}
	if (nlist != NULL)
	{
		nlist = NULL;
		memory->destroy_2d_int_array(nlist);
	}

	if (sigma != NULL)
	{
		sigma = NULL;
		memory->sfree(sigma);
	}
	if (epsilon != NULL)
	{
		epsilon = NULL;
		memory->sfree(epsilon);
	}
	if (type != NULL)
	{
		type = NULL;
		memory->sfree(type);
	}
	if (type158 != NULL)
	{
		type158 = NULL;
		memory->sfree(type158);
	}

	if (sasa != NULL)
	{
		sasa = NULL;
		memory->sfree(sasa);
	}

	if (nsalist != NULL)
	{
		nsalist = NULL;
		memory->sfree(nsalist);
	}

	if (salist != NULL)
	{
		salist = NULL;
		memory->destroy_2d_int_array(salist);
	}

	if (icontact != NULL)
	{
		icontact = NULL;
		memory->destroy_2d_int_array(icontact);
	}
	if (icontacttype != NULL)
	{
		icontacttype = NULL;
		memory->sfree(icontacttype);
	}

	if (icontact_loop != NULL)
	{
		icontact_loop = NULL;
		memory->destroy_2d_int_array(icontact_loop);
	}
	if (icontacttype_loop != NULL)
	{
		icontacttype_loop = NULL;
		memory->sfree(icontacttype_loop);
	}
	if (confidence_contact_loop != NULL)
	{
		confidence_contact_loop = NULL;
		memory->sfree(confidence_contact_loop);
	}

	if (icontactnative != NULL)
	{
		icontactnative = NULL;
		memory->destroy_2d_int_array(icontactnative);
	}

	if (nresiduecontact != NULL)
	{
		nresiduecontact = NULL;
		memory->sfree(nresiduecontact);
	}

	if (rcontact != NULL)
	{
		rcontact = NULL;
		memory->sfree(rcontact);
	}
	if (confidence_contact != NULL)
	{
		confidence_contact = NULL;
		memory->sfree(confidence_contact);
	}

	if (ncontacthelixmap != NULL)
	{
		ncontacthelixmap = NULL;
		memory->destroy_2d_int_array(ncontacthelixmap);
	}

	if (confidence_helix != NULL)
	{
		confidence_helix = NULL;
		memory->sfree(confidence_helix);
	}
	if (helixconnectionAvg != NULL)
	{
		helixconnectionAvg = NULL;
		memory->sfree(helixconnectionAvg);
	}
	if (helixconnectionDev != NULL)
	{
		helixconnectionDev = NULL;
		memory->sfree(helixconnectionDev);
	}
	if (helixangleAvg != NULL)
	{
		helixangleAvg = NULL;
		memory->destroy_2d_double_array(helixangleAvg);
	}
	if (helixangleDev != NULL)
	{
		helixangleDev = NULL;
		memory->destroy_2d_double_array(helixangleDev);
	}

	if (lst_disulfide != NULL)
	{
		lst_disulfide = NULL;
		memory->destroy_2d_int_array(lst_disulfide);
	}

	if (loopfrag != NULL)
	{
		loopfrag = NULL;
		memory->destroy_2d_int_array(loopfrag);
	}
	if (loopfrag_number != NULL)
	{
		loopfrag_number = NULL;
		memory->sfree(loopfrag_number);
	}
}
/* ---------------------------------------------------------------------- */
void Protein::init()
{
	natom1 = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:natom1");
	AtomType = memory->create_2d_char_array(MAXATOM, 8, "Protein:AtomType");
	AA3 = memory->create_2d_char_array(MAXATOM, 8, "Protein:AA3");
	naa = (int *)memory->smalloc(MAXATOM * sizeof(int), "Protein:naa");
	x = memory->create_2d_double_array(MAXATOM, 3, "Protein:x");
	x0 = memory->create_2d_double_array(MAXATOM, 3, "Protein:x0");
	x00 = memory->create_2d_double_array(MAXATOM, 3, "Protein:x00");
	phi = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:phi");
	psi = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:psi");
	nclash = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:nclash");
	tmb = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:tmb");
	tme = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:tme");
	tmi = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:tmi");
	membren_vector = (double *)memory->smalloc(3 * sizeof(double), "Protein:membren_vector");
	tmbnative = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:tmbnative");
	tmenative = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:tmenative");
	tminative = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:tminative");
	hydrophobicity = (double *)memory->smalloc(MAXAA * sizeof(double), "Protein:hydrophobicity");
	tmdv = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:tmdv");
	rwp = memory->create_3d_double_array(159, 159, 31, "Protein:rwp");
	scbp = memory->create_3d_double_array(20, 20, 2704, "Protein:scbp");

	nnlist = (int *)memory->smalloc(MAXATOM * sizeof(int), "Protein:nnlist");
	nlist = memory->create_2d_int_array(MAXATOM, 10000, "Protein:nlist");

	sigma = (double *)memory->smalloc(MAXATOM * sizeof(double), "Protein:sigma");
	epsilon = (double *)memory->smalloc(MAXATOM * sizeof(double), "Protein:epsilon");
	type = (int *)memory->smalloc(MAXATOM * sizeof(int), "Protein:type");
	type158 = (int *)memory->smalloc(MAXATOM * sizeof(int), "Protein:type158");

	sasa = (double *)memory->smalloc(MAXAA * sizeof(double), "Protein:sasa");
	nsalist = (int *)memory->smalloc(MAXATOM * sizeof(int), "Protein:nsalist");
	salist = memory->create_2d_int_array(MAXATOM, 10000, "Protein:salist");

	icontact = memory->create_2d_int_array(MAXATOM, 2, "Protein:icontact");
	icontactnative = memory->create_2d_int_array(MAXATOM, 2, "Protein:icontactnative");
	icontacttype = (CONTACTTYPE *)memory->smalloc(MAXAA * sizeof(CONTACTTYPE), "Protein:icontacttype");

	icontact_loop = memory->create_2d_int_array(MAXATOM, 2, "Protein:icontact_loop");
	icontacttype_loop = (CONTACTTYPE *)memory->smalloc(MAXAA * sizeof(CONTACTTYPE), "Protein:icontacttype_loop");
	confidence_contact_loop = (double *)memory->smalloc(MAXATOM * sizeof(double), "Protein:confidence_contact_loop");

	nresiduecontact = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:nresiduecontact");
	rcontact = (double *)memory->smalloc(MAXATOM * sizeof(double), "Protein:rcontact");
	confidence_contact = (double *)memory->smalloc(MAXATOM * sizeof(double), "Protein:confidence_contact");
	ncontacthelixmap = memory->create_2d_int_array(20, 20, "Protein:ncontacthelixmap");
	confidence_helix = (double *)memory->smalloc(MAXAA * sizeof(double), "Protein:confidence_helix");

	helixconnectionAvg = (double *)memory->smalloc(MAXAA * sizeof(double), "Protein:helixconnectionAvg");
	helixconnectionDev = (double *)memory->smalloc(MAXAA * sizeof(double), "Protein:helixconnectionDev");
	helixangleAvg = memory->create_2d_double_array(MAXAA * sizeof(double), MAXAA * sizeof(double), "Protein:helixangleAvg");
	helixangleDev = memory->create_2d_double_array(MAXAA * sizeof(double), MAXAA * sizeof(double), "Protein:helixangleDev");

	lst_disulfide = memory->create_2d_int_array(MAXATOM, 2, "Protein:lst_disulfide");

	FRAGSIZE = 5;
	MAXFRAG = 200;

	loopfrag = memory->create_2d_int_array(MAXAA, MAXFRAG, "Protein:loopfrag");
	loopfrag_number = (int *)memory->smalloc(MAXAA * sizeof(int), "Protein:loopfrag_number");
}
/* ---------------------------------------------------------------------- */
int Protein::read_pdb(char const *in)
{
	FILE *pdb;
	char junk[8], atnam[8], resnam[8], buffer[160];
	int atnum, resnum, resnum0 = 1, atnum0 = 1;
	double xx, yy, zz;

	// open input pdb file
	if ((pdb = fopen(in, "r")) == NULL)
	{
		char str[128];
		sprintf(str, "Failed to open input file: %s\n", in);
		error->error(str);
	}
	Tatom = 0;
	Taa = 0;
	natom1[Taa] = 0;
	// read input file
	while (fgets(buffer, 159, pdb))
	{
		sscanf(buffer, "%4s", junk);
		sscanf(buffer + 6, "%5d", &atnum);
		sscanf(buffer + 13, "%3s", atnam);
		sscanf(buffer + 17, "%3s", resnam);
		sscanf(buffer + 22, "%4d", &resnum);
		sscanf(buffer + 30, "%8lf", &xx);
		sscanf(buffer + 38, "%8lf", &yy);
		sscanf(buffer + 46, "%8lf", &zz);
		if (!strncmp(junk, "ATOM", 4))
		{

			Taa = resnum - 1;
			//printf("%4s  %5d  %-3s %3s  %4d    %8.3f%8.3f%8.3f\n",junk,atnum,atnam,resnam,resnum,xx,yy,zz);
			if (resnum != resnum0)
			{
				//Taa++;
				resnum0 = resnum;
				if (!strncmp(atnam, "N", 4))
				{
					natom1[Taa] = Tatom + 1;
					//printf("%d %d\n",Taa,natom1[Taa]);
				}
			}
			if (atnum != atnum0)
			{
				Tatom++;
				atnum0 = atnum;
			}
			strcpy(AtomType[Tatom], atnam);
			strcpy(AA3[Tatom], resnam);
			naa[Tatom] = Taa;
			x[Tatom][0] = (double)xx;
			x[Tatom][1] = (double)yy;
			x[Tatom][2] = (double)zz;
		}
	}
	Taa++;
	Tatom++;
	rebuild_atom_type();

	fclose(pdb);

	//print_natom1();
	return 1;
}

/* ----------------------------------------------------------------------
	 rebuild the atom type, sigma, epsilon
   ---------------------------------------------------------------------- */
void Protein::rebuild_atom_type()
{
	int i;
	for (i = 0; i < Tatom; i++)
	{
		sigma[i] = get_sigma(AA3[i], AtomType[i]);
		epsilon[i] = get_epsilon(AA3[i], AtomType[i]);
		type[i] = atom_type(AtomType[i]);
		type158[i] = atom_type_158(AA3[i], AtomType[i]);
	}
}

/* ----------------------------------------------------------------------
	 get the epsilon value for L-J potential
   ---------------------------------------------------------------------- */
double Protein::get_epsilon(char *resname, char *atomname)
{
	int i,
		j;
	for (i = 0; i <= 19; i++)
	{
		for (j = 0; j <= 35; j++)
		{
			//      fprintf(stderr,"%5s %5s",sTab3[i],resname);
			if (!strncmp(sTab3[i], resname, 8) && !strncmp(sTab4[j], atomname, 8))
			{
				return (sTabe[i][j]);
			}
		}
	}
	return (0.0);
}

/* ----------------------------------------------------------------------
	 get the sigma value for L-J potential
   ---------------------------------------------------------------------- */
double Protein::get_sigma(char *resname, char *atomname)
{
	int i, j;
	for (i = 0; i <= 19; i++)
	{
		for (j = 0; j <= 35; j++)
		{
			//      fprintf(stderr,"%5s %5s",sTab3[i],resname);
			if (!strncmp(sTab3[i], resname, 8) && !strncmp(sTab4[j], atomname, 8))
			{
				return (sTabs[i][j]);
			}
		}
	}
	return (0.0);
}

//---------------------------------------------------------------------- /
int Protein::atom_type(char *atomname)
{
	int i;
	for (i = 0; i < 36; i++)
	{
		if (!strncmp(sTab4[i], atomname, 8))
		{
			return (i);
		}
	}
	return (99);
}

//---------------------------------------------------------------------- /
int Protein::atom_type_158(char *resname, char *atomname)
{
	int i, j;
	for (i = 0; i <= 19; i++)
	{
		for (j = 0; j <= 35; j++)
		{
			//      fprintf(stderr,"%5s %5s",sTab3[i],resname);
			if (!strncmp(sTab3[i], resname, 8) && !strncmp(sTab4[j], atomname, 8))
			{
				return (Tab5[i][j]);
			}
		}
	}
	return (999);
}
